@@ -133,7 +133,7 @@ void Ilvis2Reader::initialize(PointTableRef)
     // Data are WGS84 (4326) with ITRF2000 datum (6656)
     // See http://nsidc.org/data/docs/daac/icebridge/ilvis2/index.html for
     // background
-    setSpatialReference(SpatialReference("EPSG:4326"));
+    setSpatialReference("EPSG:4326");
 }
 
 
