@@ -34,29 +34,30 @@
 
 #include <iomanip>
 
-#include <pdal/EigenUtils.hpp>
 #include <pdal/KDIndex.hpp>
 #include <pdal/PointView.hpp>
 #include <pdal/util/Algorithm.hpp>
 
+#include "private/Raster.hpp"
+
 namespace pdal
 {
 
 int PointView::m_lastId = 0;
 
 PointView::PointView(PointTableRef pointTable) : m_pointTable(pointTable),
-m_size(0), m_id(0)
+    m_layout(pointTable.layout()), m_size(0), m_id(0)
 {
 	m_id = ++m_lastId;
 }
 
 PointView::PointView(PointTableRef pointTable, const SpatialReference& srs) :
-	m_pointTable(pointTable), m_size(0), m_id(0), m_spatialReference(srs)
+	m_pointTable(pointTable), m_layout(pointTable.layout()), m_size(0),
+    m_id(0), m_spatialReference(srs)
 {
 	m_id = ++m_lastId;
 }
 
-
 PointView::~PointView()
 {}
 
@@ -73,6 +74,22 @@ PointViewIter PointView::end()
 }
 
 
+PointId PointView::tableId(PointId idx)
+{
+    if (idx > size())
+        throw pdal_error("Point index must increment.");
+    if (idx == size())
+    {
+        PointId rawId = m_pointTable.addPoint();
+        m_index.push_back(rawId);
+        m_size++;
+        assert(m_temps.empty());
+        return rawId;
+    }
+    return m_index[idx];
+}
+
+
 void PointView::setFieldInternal(Dimension::Id dim, PointId idx,
     const void *buf)
 {
@@ -100,13 +117,26 @@ void PointView::setFieldInternal(Dimension::Id dim, PointId idx,
 
 void PointView::calculateBounds(BOX2D& output) const
 {
-    pdal::calculateBounds(*this, output);
+    for (PointId idx = 0; idx < size(); idx++)
+    {
+        double x = getFieldAs<double>(Dimension::Id::X, idx);
+        double y = getFieldAs<double>(Dimension::Id::Y, idx);
+
+        output.grow(x, y);
+    }
 }
 
 
 void PointView::calculateBounds(BOX3D& output) const
 {
-    pdal::calculateBounds(*this, output);
+    for (PointId idx = 0; idx < size(); idx++)
+    {
+        double x = getFieldAs<double>(Dimension::Id::X, idx);
+        double y = getFieldAs<double>(Dimension::Id::Y, idx);
+        double z = getFieldAs<double>(Dimension::Id::Z, idx);
+
+        output.grow(x, y, z);
+    }
 }
 
 
@@ -152,6 +182,30 @@ TriangularMesh *PointView::mesh(const std::string& name)
 }
 
 
+Rasterd *PointView::createRaster(const std::string& name, const RasterLimits& limits,
+    double nodata)
+{
+    if (Utils::contains(m_rasters, name))
+        return nullptr;
+    Rasterd *r = new Rasterd(limits, name, nodata);
+    auto res = m_rasters.insert(std::make_pair(name, std::unique_ptr<Rasterd>(r)));
+    if (res.second)
+        return res.first->second.get();
+    return nullptr;
+}
+
+
+Rasterd *PointView::raster(const std::string& name)
+{
+    auto it = m_rasters.find(name);
+    if (it != m_rasters.end())
+        return it->second.get();
+    if (name.empty() && m_rasters.size())
+        return m_rasters.begin()->second.get();
+    return nullptr;
+}
+
+
 void PointView::invalidateProducts()
 {
     m_index2.reset();
