@@ -98,12 +98,9 @@ bool SpatialReference::empty() const
 
 bool SpatialReference::valid() const
 {
-    OGRScopedSpatialReference current(ogrCreateSrs(m_wkt));
-    if (!current)
-        return false;
+    OGRSpatialReference current(m_wkt.data());
 
-    OGRErr err = OSRValidate(current.get());
-    return err == OGRERR_NONE;
+    return OSRValidate(&current) == OGRERR_NONE;
 }
 
 
@@ -364,11 +361,7 @@ std::vector<int> SpatialReference::getAxisOrdering() const
 {
     std::vector<int> output;
     OGRScopedSpatialReference current = ogrCreateSrs(m_wkt);
-
-
-#if GDAL_VERSION_MAJOR >= 3
     output = current.get()->GetDataAxisToSRSAxisMapping();
-#endif
     return output;
 }
 
@@ -473,9 +466,6 @@ std::string SpatialReference::prettyWkt(const std::string& wkt)
 
 std::string SpatialReference::getWKT1() const
 {
-#if GDAL_VERSION_MAJOR < 3
-    return getWKT();
-#else
     std::string wkt = getWKT();
     if (wkt.empty())
         return wkt;
@@ -485,7 +475,8 @@ std::string SpatialReference::getWKT1() const
     if (srs)
     {
         char *buf = nullptr;
-        const char* apszOptions[] = { "FORMAT=WKT1_GDAL", nullptr };
+        const char* apszOptions[] =
+            { "FORMAT=WKT1_GDAL", "ALLOW_ELLIPSOIDAL_HEIGHT_AS_VERTICAL_CRS=YES", nullptr };
 
         srs->exportToWkt(&buf, apszOptions);
         if (buf)
@@ -495,10 +486,8 @@ std::string SpatialReference::getWKT1() const
         }
     }
     if (wkt1.empty())
-        throw pdal_error("Couldn't convert spatial reference to WKT "
-            "version 1.");
+        throw pdal_error("Couldn't convert spatial reference to WKT version 1.");
     return wkt1;
-#endif
 }
 
 
