@@ -148,6 +148,22 @@ std::ostream *createFile(std::string const& name, bool asBinary)
 }
 
 
+std::ostream *openExisting(const std::string& name, bool asBinary)
+{
+    std::ios::openmode mode = std::ios::out | std::ios::in;
+    if (asBinary)
+        mode |= std::ios::binary;
+
+    std::ostream *ofs = new std::ofstream(toNative(name), mode);
+    if (!ofs->good())
+    {
+        delete ofs;
+        return nullptr;
+    }
+    return ofs;
+}
+
+
 bool directoryExists(const std::string& dirname)
 {
     //ABELL - Seems we should be calling is_directory
@@ -253,9 +269,14 @@ bool fileExists(const std::string& name)
 }
 
 
+/// \return  0 on error or invalid file type.
 uintmax_t fileSize(const std::string& file)
 {
-    return pdalboost::filesystem::file_size(toNative(file));
+    pdalboost::system::error_code ec;
+    uintmax_t size = pdalboost::filesystem::file_size(toNative(file), ec);
+    if (ec)
+        size = 0;
+    return size;
 }
 
 
@@ -449,8 +470,7 @@ std::vector<std::string> glob(std::string path)
 }
 
 
-MapContext mapFile(const std::string& filename, bool readOnly,
-    size_t pos, size_t size)
+MapContext mapFile(const std::string& filename, bool readOnly, uintmax_t pos, uintmax_t size)
 {
     MapContext ctx;
 
@@ -460,7 +480,17 @@ MapContext mapFile(const std::string& filename, bool readOnly,
         return ctx;
     }
 
-#ifndef WIN32
+    if (size == 0)
+    {
+        size = FileUtils::fileSize(filename);
+        if (size == 0)
+        {
+            ctx.m_error = "File doesn't exist or isn't a regular file. Perhaps provide a size?";
+            return ctx;
+        }
+    }
+
+#ifndef _WIN32
     ctx.m_fd = ::open(filename.c_str(), readOnly ? O_RDONLY : O_RDWR);
 #else
     ctx.m_fd = ::_open(filename.c_str(), readOnly ? O_RDONLY : O_RDWR);
@@ -484,7 +514,7 @@ MapContext mapFile(const std::string& filename, bool readOnly,
     ctx.m_handle = CreateFileMapping((HANDLE)_get_osfhandle(ctx.m_fd),
         NULL, PAGE_READONLY, 0, 0, NULL);
     uint32_t low = pos & 0xFFFFFFFF;
-    uint32_t high = (pos >> 8);
+    uint32_t high = (uint32_t)(pos >> 8);
     ctx.m_addr = MapViewOfFile(ctx.m_handle, FILE_MAP_READ, high, low,
         ctx.m_size);
     if (ctx.m_addr == nullptr)
