@@ -77,6 +77,38 @@ namespace Utils
     const char pathListSeparator = ':';
 #endif
 
+    class StatusWithReason
+    {
+    public:
+        StatusWithReason() : m_code(0)
+        {}
+        StatusWithReason(bool ok)
+        {
+            if (ok)
+                m_code = 0;
+            else
+                m_code = -1;
+        }
+        StatusWithReason(int code);  // Not defined
+        StatusWithReason(int code, const std::string& what) :
+            m_code(code), m_what(what)
+        {}
+
+        int code() const
+        { return m_code; }
+
+        operator bool () const
+        { return (m_code == 0); }
+
+        std::string what() const
+        { return m_what; }
+
+    private:
+        int m_code;
+        std::string m_what;
+    };
+
+
     /**
      * \brief Clamp value to given bounds.
      *
@@ -109,27 +141,6 @@ namespace Utils
     */
     PDAL_DLL double random(double minimum, double maximum);
 
-    /**
-      Generate values in a uniform distribution in the range [minimum, maximum]
-      using the provided seed value.
-
-      \param double  Lower value of range for random number generation.
-      \param double  Upper value of range for random number generation.
-      \param seed    Seed value for random number generation.
-    */
-    PDAL_DLL double uniform(const double& minimum, const double& maximum,
-        uint32_t seed);
-    /**
-      Generate values in a normal distribution in the range [minimum, maximum]
-      using the provided seed value.
-
-      \param double  Lower value of range for random number generation.
-      \param double  Upper value of range for random number generation.
-      \param seed    Seed value for random number generation.
-    */
-    PDAL_DLL double normal(const double& mean, const double& sigma,
-        uint32_t seed);
-
     /**
       Determine if two values are within a particular range of each other.
 
@@ -243,8 +254,8 @@ namespace Utils
     /**
       Fetch the value of an environment variable.
 
-      \param name  Name of environment varaible.
-      \param name  Value of the environemnt variable if it exists, empty
+      \param name  Name of environment variable.
+      \param name  Value of the environment variable if it exists, empty
         otherwise.
       \return  0 on success, -1 on failure
     */
@@ -275,14 +286,14 @@ namespace Utils
     PDAL_DLL void eatwhitespace(std::istream& s);
 
     /**
-      Remove whitspace from the beginning of a string.
+      Remove whitespace from the beginning of a string.
 
       \param s  String to be trimmed.
     */
     PDAL_DLL void trimLeading(std::string& s);
 
     /**
-      Remove whitspace from the end of a string.
+      Remove whitespace from the end of a string.
 
       \param s  String to be trimmed.
     */
@@ -926,7 +937,7 @@ namespace Utils
 
 
     template<typename T>
-    bool fromString(const std::string& from, T* & to)
+    StatusWithReason fromString(const std::string& from, T* & to)
     {
         void *v;
         // Uses sscanf instead of operator>>(istream, void*&) as a workaround
@@ -948,7 +959,7 @@ namespace Utils
       \return  \c true if the conversion was successful, \c false otherwise.
     */
     template<typename T>
-    bool fromString(const std::string& from, T& to)
+    StatusWithReason fromString(const std::string& from, T& to)
     {
         std::istringstream iss(from);
 
@@ -958,7 +969,7 @@ namespace Utils
 
     // Optimization of above.
     template<>
-    inline bool fromString(const std::string& from, std::string& to)
+    inline StatusWithReason fromString(const std::string& from, std::string& to)
     {
         to = from;
         return true;
@@ -972,7 +983,7 @@ namespace Utils
       \return  \c true if the conversion was successful, \c false otherwise.
     */
     template<>
-    inline bool fromString<char>(const std::string& s, char& to)
+    inline StatusWithReason fromString(const std::string& s, char& to)
     {
         try
         {
@@ -1003,8 +1014,7 @@ namespace Utils
       \return  \c true if the conversion was successful, \c false otherwise.
     */
     template<>
-    inline bool fromString<unsigned char>(const std::string& s,
-        unsigned char& to)
+    inline StatusWithReason fromString(const std::string& s, unsigned char& to)
     {
         try
         {
@@ -1036,7 +1046,7 @@ namespace Utils
       \return  \c true if the conversion was successful, \c false otherwise.
     */
     template<>
-    inline bool fromString<signed char>(const std::string& s, signed char& to)
+    inline StatusWithReason fromString(const std::string& s, signed char& to)
     {
         try
         {
@@ -1067,7 +1077,7 @@ namespace Utils
       \return  \c true if the conversion was successful, \c false otherwise.
     */
     template<>
-    inline bool fromString<double>(const std::string& s, double& d)
+    inline StatusWithReason fromString(const std::string& s, double& d)
     {
         if (s == "nan" || s == "NaN")
         {
