@@ -35,12 +35,19 @@
 #include <pdal/pdal_test_main.hpp>
 
 #include <sstream>
-
-#include <pdal/util/Utils.hpp>
-
 #include <vector>
 
-using namespace pdal;
+#include <pdal/util/portable_endian.hpp>
+#include <pdal/util/FileUtils.hpp>
+#include <pdal/util/Utils.hpp>
+#include "Support.hpp"
+
+#ifdef _WIN32
+#include <winioctl.h>
+#endif
+
+namespace pdal
+{
 
 TEST(UtilsTest, test_random)
 {
@@ -122,8 +129,6 @@ TEST(UtilsTest, test_base64)
 
     EXPECT_EQ(decoded.size(), data.size());
     EXPECT_EQ(size, begin_size);
-
-    EXPECT_THROW(Utils::base64_decode("Thisisatest"), std::runtime_error);
 }
 
 TEST(UtilsTest, blanks)
@@ -471,3 +476,65 @@ TEST(UtilsTest, escapeJSON)
     std::string escaped = Utils::escapeJSON("\u0001\t\f\n\\\"\u0016");
     EXPECT_EQ(escaped, "\\u0001\\t\\f\\n\\\\\\\"\\u0016");
 }
+
+TEST(UtilsTest, map)
+{
+    Support::Tempfile temp;
+
+    std::string filename = temp.filename();
+
+    std::ostream *out;
+    // This turns on sparse file support. Otherwise, we're going to make a huge
+    // file that won't fit on many filesystems and an error will occur. If we
+    // can't set the file to sparse, we just return.  UNIX filesystems I'm
+    // aware of support sparse files without this mess.
+#ifdef _WIN32
+    auto f = CreateFileA(filename.data(), GENERIC_READ | GENERIC_WRITE,
+        0, NULL, CREATE_NEW, FILE_ATTRIBUTE_NORMAL, NULL);
+    DWORD flags;
+    GetVolumeInformationByHandleW(f, NULL, 0, NULL, NULL, &flags, NULL, 0);
+    bool ok = false;
+    if (flags & FILE_SUPPORTS_SPARSE_FILES)
+    {
+        DWORD tmp;
+        ok = DeviceIoControl(f, FSCTL_SET_SPARSE, NULL, 0, NULL, 0, &tmp, NULL);
+    }
+    CloseHandle(f);
+    if (!ok)
+        return;
+    out = FileUtils::openExisting(filename);
+#else
+    out = FileUtils::createFile(filename);
+#endif
+
+    out->seekp(50000);
+    *out << 1234;
+    out->write("Test", 4);
+    out->seekp(0x10FFFFFFFF);
+    *out << 5678;
+    out->write("Another.", 9);
+    FileUtils::closeFile(out);
+
+    auto ctx = FileUtils::mapFile(filename);
+    assert(ctx.addr());
+    char *c = reinterpret_cast<char *>(ctx.addr()) + 50000;
+
+    EXPECT_EQ(*c++, '1');
+    EXPECT_EQ(*c++, '2');
+    EXPECT_EQ(*c++, '3');
+    EXPECT_EQ(*c++, '4');
+    EXPECT_EQ(*c++, 'T');
+    EXPECT_EQ(*c++, 'e');
+    EXPECT_EQ(*c++, 's');
+    EXPECT_EQ(*c++, 't');
+
+    c = reinterpret_cast<char *>(ctx.addr()) + 0x10FFFFFFFF;
+    EXPECT_EQ(*c++, '5');
+    EXPECT_EQ(*c++, '6');
+    EXPECT_EQ(*c++, '7');
+    EXPECT_EQ(*c++, '8');
+    EXPECT_EQ(std::string(c), "Another.");
+    FileUtils::unmapFile(ctx);
+}
+
+}
