@@ -173,6 +173,27 @@ TEST_F(TransformationFilterTest, Translation)
 }
 
 
+TEST_F(TransformationFilterTest, InvertTranslation)
+{
+    Options filterOpts;
+    filterOpts.add("matrix", "1 0 0 1\n0 1 0 2\n0 0 1 3\n0 0 0 1");
+    filterOpts.add("invert", true);
+    m_filter.setOptions(filterOpts);
+
+    PointTable table;
+    m_filter.prepare(table);
+    PointViewSet viewSet = m_filter.execute(table);
+    PointViewPtr view = *viewSet.begin();
+
+    for (point_count_t i = 0; i < view->size(); ++i)
+    {
+        EXPECT_DOUBLE_EQ(0, view->getFieldAs<double>(Dimension::Id::X, i));
+        EXPECT_DOUBLE_EQ(0, view->getFieldAs<double>(Dimension::Id::Y, i));
+        EXPECT_DOUBLE_EQ(0, view->getFieldAs<double>(Dimension::Id::Z, i));
+    }
+}
+
+
 TEST_F(TransformationFilterTest, Rotation)
 {
     Options filterOpts;
@@ -192,6 +213,30 @@ TEST_F(TransformationFilterTest, Rotation)
     }
 }
 
+
+TEST_F(TransformationFilterTest, InvertRotation)
+{
+    Options filterOpts;
+    filterOpts.add("matrix", "0 1 0 0\n-1 0 0 0\n0 0 1 0\n0 0 0 1");
+    filterOpts.add("invert", true);
+    m_filter.setOptions(filterOpts);
+
+    PointTable table;
+    m_filter.prepare(table);
+    PointViewSet viewSet = m_filter.execute(table);
+    EXPECT_EQ(1u, viewSet.size());
+    PointViewPtr view = *viewSet.begin();
+    EXPECT_EQ(3u, view->size());
+
+    for (point_count_t i = 0; i < view->size(); ++i)
+    {
+        EXPECT_DOUBLE_EQ(-2, view->getFieldAs<double>(Dimension::Id::X, i));
+        EXPECT_DOUBLE_EQ(1, view->getFieldAs<double>(Dimension::Id::Y, i));
+        EXPECT_DOUBLE_EQ(3, view->getFieldAs<double>(Dimension::Id::Z, i));
+    }
+}
+
+
 TEST_F(TransformationFilterTest, SrsReset)
 {
     TransformationFilter::Transform ident { { 1, 0, 0, 0,
@@ -209,7 +254,7 @@ TEST_F(TransformationFilterTest, SrsReset)
     PointViewSet viewSet = m_filter.execute(table);
     PointViewPtr view = *viewSet.begin();
 
-    EXPECT_EQ(view->spatialReference(), SpatialReference("EPSG:3857"));
+    EXPECT_EQ(view->spatialReference(), "EPSG:3857");
 }
 
 TEST(TransformationMatrix, init_file_oneline)
