@@ -288,42 +288,6 @@ TEST(EptReaderTest, resolutionLimit)
     EXPECT_EQ(np, expectedCount);
 }
 
-TEST(EptReaderTest, bounds2dXform)
-{
-    BOX2D b(515380, 4918360, 515390, 4918370);
-    SrsBounds eptBounds(b);
-    gdal::reprojectBounds(b, "EPSG:26912", "EPSG:4326");
-    SrsBounds boxBounds(b, "EPSG:4326");
-
-    PointViewPtr v1;
-    PointViewPtr v2;
-    {
-        EptReader reader;
-        Options options;
-        options.add("filename", eptLaszipPath);
-        options.add("bounds", eptBounds);
-        reader.setOptions(options);
-        PointTable eptTable;
-        reader.prepare(eptTable);
-        auto vset = reader.execute(eptTable);
-        v1 = *vset.begin();
-    }
-    {
-        EptReader reader;
-        Options options;
-        options.add("filename", eptLaszipPath);
-        options.add("bounds", boxBounds);
-        reader.setOptions(options);
-        PointTable eptTable;
-        reader.prepare(eptTable);
-        auto vset = reader.execute(eptTable);
-        v2 = *vset.begin();
-    }
-
-    // There is some small error when we round-trip the bounds, so allow us
-    // to be off by 15 points.
-    EXPECT_NEAR(v1->size(), v2->size(), 15u);
-}
 
 TEST(EptReaderTest, boundedRead2d)
 {
@@ -622,8 +586,7 @@ TEST(EptReaderTest, boundedCrop)
     {
         Options options;
         options.add("filename", eptAutzenPath);
-        std::string overrides(wkt + "/ EPSG:3644");
-        Option polygon("polygon", overrides);
+        Option polygon("polygon", wkt + "/ EPSG:3644");
         options.add(polygon);
         reader.setOptions(options);
     }
@@ -647,9 +610,8 @@ TEST(EptReaderTest, boundedCrop)
     }
     CropFilter crop;
     {
-        std::string overrides(wkt + "/ EPSG:3644");
         Options options;
-        Option polygon("polygon", overrides);
+        Option polygon("polygon", wkt + "/ EPSG:3644");
         options.add(polygon);
         crop.setOptions(options);
         crop.setInput(source);
@@ -734,7 +696,6 @@ TEST(EptReaderTest, boundedCropReprojection)
 
 TEST(EptReaderTest, ogrCrop)
 {
-
     EptReader reader;
     {
         Options options;
@@ -754,9 +715,7 @@ TEST(EptReaderTest, ogrCrop)
 
     uint64_t eptNp(0);
     for (const PointViewPtr& view : reader.execute(eptTable))
-    {
         eptNp += view->size();
-    }
 
     // Now we'll check the result against a crop filter of the source file with
     // the same bounds.
@@ -770,15 +729,11 @@ TEST(EptReaderTest, ogrCrop)
     source.prepare(sourceTable);
     uint64_t sourceNp(0);
     for (const PointViewPtr& view : source.execute(sourceTable))
-    {
         sourceNp += view->size();
-    }
 
     EXPECT_EQ(eptNp, sourceNp);
     EXPECT_EQ(eptNp, 86u);
     EXPECT_EQ(sourceNp, 86u);
 }
 
-
-
 } // namespace pdal
