@@ -46,7 +46,6 @@
 #include <io/LasReader.hpp>
 #include <io/LasWriter.hpp>
 #include <io/BpfReader.hpp>
-
 #include "Support.hpp"
 
 namespace pdal
@@ -69,6 +68,7 @@ public:
 
 using namespace pdal;
 
+
 TEST(LasWriterTest, srs)
 {
     Options readerOps;
@@ -176,14 +176,11 @@ TEST(LasWriterTest, auto_offset)
     reader.setOptions(readerOps);
 
     reader.prepare(readTable);
-    EXPECT_DOUBLE_EQ(25.00, reader.header().offsetX());
+    EXPECT_DOUBLE_EQ((1000000.02 + 25) * 0.5, reader.header().offsetX());
     EXPECT_DOUBLE_EQ(0, reader.header().offsetY());
-    EXPECT_DOUBLE_EQ(-123, reader.header().offsetZ());
+    EXPECT_DOUBLE_EQ(-123 * 0.5 + 2147483524 * 0.5, reader.header().offsetZ());
 
-    EXPECT_DOUBLE_EQ(0.00046564965530561733, reader.header().scaleX());
-    EXPECT_DOUBLE_EQ(.01, reader.header().scaleY());
     // (max - min) are chosen to yield std::numeric_limits<int>::max();
-    EXPECT_DOUBLE_EQ(1.0, reader.header().scaleZ());
 
     PointViewSet viewSet = reader.execute(readTable);
     EXPECT_EQ(viewSet.size(), 1u);
@@ -195,10 +192,11 @@ TEST(LasWriterTest, auto_offset)
     EXPECT_NEAR(25.00, view->getFieldAs<double>(Id::X, 3), .0001);
     EXPECT_NEAR(74529.00, view->getFieldAs<double>(Id::X, 4), .001);
     EXPECT_NEAR(534252.35, view->getFieldAs<double>(Id::X, 5), .0001);
-
+    EXPECT_NEAR(2147483524, view->getFieldAs<double>(Id::Z, 4), .0001);
     FileUtils::deleteFile(FILENAME);
 }
 
+
 // Identical to above, but writes each input view to a separate output file.
 TEST(LasWriterTest, auto_offset2)
 {
@@ -260,14 +258,14 @@ TEST(LasWriterTest, auto_offset2)
         reader.setOptions(readerOps);
 
         reader.prepare(readTable);
-        EXPECT_DOUBLE_EQ(74529.00, reader.header().offsetX());
+        EXPECT_DOUBLE_EQ((74529.00 + 1000000.02) * 0.5, reader.header().offsetX());
         EXPECT_DOUBLE_EQ(0, reader.header().offsetY());
-        EXPECT_DOUBLE_EQ(-123, reader.header().offsetZ());
+        EXPECT_DOUBLE_EQ((-123 + 945.23) * 0.5, reader.header().offsetZ());
 
-        EXPECT_NEAR(4.30956e-4, reader.header().scaleX(), 1e-4);
+        EXPECT_NEAR(2.15478e-4, reader.header().scaleX(), 1e-4);
         EXPECT_DOUBLE_EQ(.01, reader.header().scaleY());
         // (max - min) are chosen to yield std::numeric_limits<int>::max();
-        EXPECT_NEAR(4.9743e-7, reader.header().scaleZ(), 1e-7);
+        EXPECT_NEAR(2.48716e-7, reader.header().scaleZ(), 1e-7);
 
         PointViewSet viewSet = reader.execute(readTable);
         EXPECT_EQ(viewSet.size(), 1u);
@@ -287,13 +285,13 @@ TEST(LasWriterTest, auto_offset2)
         reader.setOptions(readerOps);
 
         reader.prepare(readTable);
-        EXPECT_DOUBLE_EQ(25.0, reader.header().offsetX());
+        EXPECT_NEAR((534252.35 + 25.0) * 0.5, reader.header().offsetX(), 0.01);
         EXPECT_DOUBLE_EQ(0, reader.header().offsetY());
-        EXPECT_DOUBLE_EQ(1.5, reader.header().offsetZ());
+        EXPECT_NEAR((2147483524 + 1.5) * 0.5, reader.header().offsetZ(), 0.01);
 
-        EXPECT_NEAR(2.4876e-4, reader.header().scaleX(), 1e-7);
+        EXPECT_NEAR(1.2438e-4, reader.header().scaleX(), 1e-7);
         EXPECT_DOUBLE_EQ(.01, reader.header().scaleY());
-        EXPECT_NEAR(.99999, reader.header().scaleZ(), 1e-5);
+        EXPECT_NEAR(.49999, reader.header().scaleZ(), 1e-5);
 
         PointViewSet viewSet = reader.execute(readTable);
         EXPECT_EQ(viewSet.size(), 1u);
@@ -302,6 +300,7 @@ TEST(LasWriterTest, auto_offset2)
         EXPECT_NEAR(25.00, view->getFieldAs<double>(Id::X, 0), .0001);
         EXPECT_NEAR(74529.00, view->getFieldAs<double>(Id::X, 1), .001);
         EXPECT_NEAR(534252.35, view->getFieldAs<double>(Id::X, 2), .0001);
+        EXPECT_NEAR(2147483524, view->getFieldAs<double>(Id::Z, 1), .0001);
     }
 
     FileUtils::deleteFile(inname1);
@@ -309,6 +308,83 @@ TEST(LasWriterTest, auto_offset2)
 }
 
 
+// test autoscale with autooffset for a dataset with 0 variance in at least one
+// of it's spatial dimensions (e.g. points sampled from an axis alligned plane)
+TEST(LasWriterTest, auto_scale_with_auto_offset)
+{
+    using namespace Dimension;
+
+    const std::string FILENAME(
+        Support::temppath("autoscale_with_autooffset_test.las"));
+    PointTable table;
+
+    table.layout()->registerDims({Id::X, Id::Y, Id::Z});
+
+    BufferReader bufferReader;
+
+    // in 'Z' there is 0 variance which causes PDAL LasWriter to fail (scale is 0)
+    PointViewPtr view(new PointView(table));
+    // first point
+    view->setField(Id::X, 0, 780.00);
+    view->setField(Id::Y, 0, 240.00);
+    view->setField(Id::Z, 0, 25.00);
+    // second point
+    view->setField(Id::X, 1, 400.00);
+    view->setField(Id::Y, 1, 120.00);
+    view->setField(Id::Z, 1, 25.00);
+    // third point
+    view->setField(Id::X, 2, -250.00);
+    view->setField(Id::Y, 2, -660.76);
+    view->setField(Id::Z, 2, 25.00);
+
+    bufferReader.addView(view);
+
+    Options writerOps;
+    writerOps.add("filename", FILENAME);
+    writerOps.add("offset_x", "auto");
+    writerOps.add("offset_y", "auto");
+    writerOps.add("offset_z", "auto");
+    writerOps.add("scale_x", "auto");
+    writerOps.add("scale_y", "auto");
+    writerOps.add("scale_z", "auto");
+
+    LasWriter writer;
+    writer.setOptions(writerOps);
+    writer.setInput(bufferReader);
+
+    writer.prepare(table);
+    writer.execute(table);
+
+    Options readerOps;
+    readerOps.add("filename", FILENAME);
+
+    PointTable readTable;
+
+    LasReader reader;
+    reader.setOptions(readerOps);
+
+    reader.prepare(readTable);
+    EXPECT_DOUBLE_EQ((-250.00 + 780.00) * 0.5, reader.header().offsetX());
+    PointViewSet viewSet = reader.execute(readTable);
+    EXPECT_EQ(viewSet.size(), 1u);
+    view = *viewSet.begin();
+    EXPECT_EQ(view->size(), 3u);
+    // test first point
+    EXPECT_NEAR(780.00, view->getFieldAs<double>(Id::X, 0), .0001);
+    EXPECT_NEAR(240.00, view->getFieldAs<double>(Id::Y, 0), .0001);
+    EXPECT_NEAR(25.00, view->getFieldAs<double>(Id::Z, 0), .0001);
+    // test second point
+    EXPECT_NEAR(400.00, view->getFieldAs<double>(Id::X, 1), .0001);
+    EXPECT_NEAR(120.00, view->getFieldAs<double>(Id::Y, 1), .0001);
+    EXPECT_NEAR(25.00, view->getFieldAs<double>(Id::Z, 1), .0001);
+    // test third point
+    EXPECT_NEAR(-250.00, view->getFieldAs<double>(Id::X, 2), .0001);
+    EXPECT_NEAR(-660.76, view->getFieldAs<double>(Id::Y, 2), .0001);
+    EXPECT_NEAR(25.00, view->getFieldAs<double>(Id::Z, 2), .0001);
+    FileUtils::deleteFile(FILENAME);
+}
+
+
 TEST(LasWriterTest, extra_dims)
 {
     Options readerOps;
@@ -716,13 +792,13 @@ TEST(LasWriterTest, lazperf)
     EXPECT_EQ(view1->pointSize(), view2->pointSize());
 
    // Validate some point data.
-    std::unique_ptr<char> buf1(new char[pointSize]);
-    std::unique_ptr<char> buf2(new char[pointSize]);
+    std::vector<char> buf1(pointSize);
+    std::vector<char> buf2(pointSize);
     for (PointId i = 0; i < view1->pointSize(); i += 100)
     {
-       view1->getPackedPoint(dims, i, buf1.get());
-       view2->getPackedPoint(dims, i, buf2.get());
-       EXPECT_EQ(memcmp(buf1.get(), buf2.get(), pointSize), 0);
+       view1->getPackedPoint(dims, i, buf1.data());
+       view2->getPackedPoint(dims, i, buf2.data());
+       EXPECT_EQ(memcmp(buf1.data(), buf2.data(), pointSize), 0);
     }
 }
 #endif
@@ -784,13 +860,13 @@ TEST(LasWriterTest, laszip)
     EXPECT_EQ(view1->pointSize(), view2->pointSize());
 
    // Validate some point data.
-    std::unique_ptr<char> buf1(new char[pointSize]);
-    std::unique_ptr<char> buf2(new char[pointSize]);
+    std::vector<char> buf1(pointSize);
+    std::vector<char> buf2(pointSize);
     for (PointId i = 0; i < view1->size(); i += 100)
     {
-       view1->getPackedPoint(dims, i, buf1.get());
-       view2->getPackedPoint(dims, i, buf2.get());
-       EXPECT_EQ(memcmp(buf1.get(), buf2.get(), pointSize), 0);
+       view1->getPackedPoint(dims, i, buf1.data());
+       view2->getPackedPoint(dims, i, buf2.data());
+       EXPECT_EQ(memcmp(buf1.data(), buf2.data(), pointSize), 0);
     }
 }
 
@@ -854,14 +930,14 @@ TEST(LasWriterTest, laszip1_4)
     EXPECT_EQ(view1->pointSize(), view2->pointSize());
 
    // Validate some point data.
-    std::unique_ptr<char> buf1(new char[pointSize]);
-    std::unique_ptr<char> buf2(new char[pointSize]);
+    std::vector<char> buf1(pointSize);
+    std::vector<char> buf2(pointSize);
     for (PointId idx = 0; idx < view1->size(); idx++)
     {
-       view1->getPackedPoint(dims, idx, buf1.get());
-       view2->getPackedPoint(dims, idx, buf2.get());
-       char *b1 = buf1.get();
-       char *b2 = buf2.get();
+       view1->getPackedPoint(dims, idx, buf1.data());
+       view2->getPackedPoint(dims, idx, buf2.data());
+       char *b1 = buf1.data();
+       char *b2 = buf2.data();
        // Uncomment this to figure out the exact byte at which things are
        // broken.
        /**
@@ -882,7 +958,7 @@ TEST(LasWriterTest, laszip1_4)
            }
        }
        **/
-       EXPECT_EQ(memcmp(buf1.get(), buf2.get(), pointSize), 0);
+       EXPECT_EQ(memcmp(buf1.data(), buf2.data(), pointSize), 0);
     }
 }
 
@@ -1308,7 +1384,7 @@ TEST(LasWriterTest, badVlr)
     );
 }
 
-TEST(LasWriterTest, evlroffset)
+TEST(LasWriterTest, evlrOffset)
 {
     std::string outfile(Support::temppath("evlr.las"));
 
@@ -1507,6 +1583,35 @@ TEST(LasWriterTest, issue2663)
     EXPECT_EQ(h.scaleZ(), .001);
 }
 
+// Make sure that we don't crash when writing scan angle values >90 && <-90 in point
+// formats 6 or higher.
+TEST(LasWriterTest, issue3288)
+{
+    point_count_t cnt;
+
+    std::string infile(Support::datapath("text/largescan.txt"));
+    std::string tmpfile(Support::temppath("out.laz"));
+    FileUtils::deleteFile(tmpfile);
+    {
+        Options wopts;
+        wopts.add("dataformat_id", 6);
+        wopts.add("minor_version", 4);
+
+        PipelineManager mgr;
+        Stage& r = mgr.makeReader(infile, "readers.text");
+
+        mgr.makeWriter(tmpfile, "writers.las", r, wopts);
+        cnt = mgr.execute();
+    }
+
+    {
+        PipelineManager mgr;
+        mgr.makeReader(tmpfile, "readers.las");
+        EXPECT_EQ(cnt, mgr.execute());
+    }
+}
+
+
 #if defined(PDAL_HAVE_LASZIP)
 // Make sure that we can translate this special test data to 1.4, dataformat 6.
 TEST(LasWriterTest, issue2320)
